/***************************************************************************
    File:         utils.h
    Project:      Kio-Sword -- An ioslave for SWORD and KDE
    Copyright:    Copyright (C) 2004-2005 Luke Plant
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef UTILS_H
#define UTILS_H

#include <ntqstring.h>
namespace KioSword {

	class SwordOptions;

	typedef enum {  DEFMODULETYPE_NONE,
			DEFBIBLE,
			GREEKSTRONGS,
			HEBREWSTRONGS,
			GREEKMORPH,
			HEBREWMORPH } DefModuleType;
	
	extern const char* DEFBIBLE_STR;
	extern const char* GREEKSTRONGS_STR;
	extern const char* HEBREWSTRONGS_STR;
	extern const char* GREEKMORPH_STR;
	extern const char* HEBREWMORPH_STR;
	
	TQString optionsAsHiddenFields(const SwordOptions& options);
	TQString swordUrl(const TQString& path, const SwordOptions& options, bool htmlEncodeOutput = true);
	TQString swordUrl(const TQString& module, const TQString& ref, const SwordOptions& options, bool htmlEncodeOutput = true);
	TQString swordUrlForPage(const TQString& page, const SwordOptions& options, bool htmlEncodeOutput = true);
	TQString swordUrlForSettings(const TQString& path, const SwordOptions& options, bool htmlEncodeOutput = true);
	TQString swordUrlForSearch(DefModuleType modType, const TQString& searchQuery, const SwordOptions& options, bool htmlEncodeOutput = true);
	TQString swordUrlForSearch(DefModuleType modType, const TQString& searchQuery, const SwordOptions* options, bool htmlEncodeOutput = true);
	TQString shorten(const TQString& ref, uint len);

}
#endif
